%define ALD_Ver		1.2.4

#USE gcc-4.9.4

%define OrgName ncurses
Name:         uC-%{OrgName}

License:      GPL
Summary:      GNU The uClibc ncurses-5.7 libraries
Group:        Development/Libraries
Version:      5.7
Release:      1025.0

Source: %{OrgName}-%{version}.tar.xz
Source1: CROSS-uC.sh
Patch0: %{OrgName}-%{version}.patch

%define uC_Version 1.0.25
%define DestDir		/Embedded/uC-%{uC_Version}

# -----------------------------------------------------
%if %{_target_cpu} == %{_build_cpu}
%define _CROSS 0
%define BUILD_LOCAL_CROSS 0
%else
# rpm --target=x86_64 -ba xxxxx.spec
%define _CROSS 1
%define BUILD_LOCAL_CROSS 1
%endif

%define TO_ARCH %{_target_cpu}

%if %{_target_cpu} == "armv6l"
%define ARM_ARCH armv6zk
%define ARM_FPU  -mfpu=vfp -mfloat-abi=hard
%define TO_ARCH %{ARM_ARCH}
%endif

%if %{_target_cpu} == "armv7l"
%define ARM_ARCH armv7-a
%define ARM_FPU  -mfpu=vfpv3-d16 -mfloat-abi=hard
%define TO_ARCH %{ARM_ARCH}
%endif

%define BUILD_TAR 1
# -----------------------------------------------------

%define _prefix /usr/uC/%{uC_Version}/static
%define _lib lib

#------------------ CROSS DIR --------------------
%if %{_CROSS}

%if %{_os} == "mingw"
ERROR: target=%{_os}
%else  #win32 || win64
%define target %{_arch}-%{_os}%{_gnu}
%endif

%define crossdir	/cross
%if %{_arch} == "arm"
%define csysdir		%{crossdir}/%{ARM_ARCH}
%else
%define csysdir		%{crossdir}/%{_target_cpu}
%endif
%define csysbindir	%{csysdir}/usr/uC/%{uC_Version}/static/bin
%define target_  %{target}-

%define __os_install_post %{nil}

%define cross_bindir	%{csysdir}%{_bindir}
%define cross_sbindir	%{csysdir}%{_sbindir}
%define cross_libdir	%{csysdir}%{_libdir}
%define cross_includedir %{csysdir}%{_includedir}
%define cross_datadir	%{csysdir}%{_datadir}
%define cross_infodir	%{csysdir}%{_infodir}
%define cross_mandir	%{csysdir}%{_mandir}
%define cross_defaultdocdir %{csysdir}%{_defaultdocdir}
%define cross_docdir	%{cross_defaultdocdir}/%{OrgName}

%define CrossDestDir	/Cross

%define _srcrpmdir %{_topdir}/SRPMS/tmp

%else  #_CROSS

%define csysbindir	%{_prefix}/bin
%define CrossDestDir %{nil}
%define _srcrpmdir %{_topdir}/SRPMS/%{ALD_Ver}

%define target %{_arch}-uclibc
%define target_  %{nil}

%endif # !_CROSS

%define docdir %{_defaultdocdir}/%{OrgName}

# ------------------- description ------------------------------------
%description
GNU The uClibc ncurses-5.7 libraries
# -------------------
%package bin
Summary:      %{OrgName} utilities static uClibc
Group:        System Environment/Base
%description bin
%{OrgName} utilities static uClibc
#-------------------------------
%package %{TO_ARCH}
Summary:      %{name} Cross-%{TO_ARCH} Libraries
Group:        Development/Libraries
%description %{TO_ARCH}
%{name} Cross-%{TO_ARCH} Libraries

# ------------------- setup ------------------------------------------
%Prep
%setup -q -n %{OrgName}-%{version}
%if %{_CROSS}
%patch0 -b .orig -p0
%endif
# ------------------- Build ------------------------------------------
%Build
PATH=%{csysbindir}:$PATH:/opt/perl/bin
GCC_NoHostPath=d
UCLIBC_GCC_INC_SKIP=%{csysdir}/usr/include
export PATH GCC_NoHostPath UCLIBC_GCC_INC_SKIP

GCC_VER_STR=`%{target_}gcc -dumpversion`

%if %{_arch} == "i386"
OPTFLAGS="-Os -march=i486 -mpreferred-stack-boundary=2 -falign-functions=1 -falign-jumps=1 -falign-loops=1 -fomit-frame-pointer"
OPT47=`case $GCC_VER_STR in 4.[789].*) echo "-fno-asynchronous-unwind-tables" ;; esac`
%endif
%if %{_arch} == "x86_64"
OPTFLAGS="-Os"
OPT47=`case $GCC_VER_STR in 4.[789].*) echo "-fno-asynchronous-unwind-tables" ;; esac`
%endif
%if %{_arch} == "arm"
OPTFLAGS="-Os -march=%{ARM_ARCH} %{ARM_FPU} -munaligned-access"
OPT47=
%endif

#CFLAGS="$OPTFLAGS $OPT47 -ffunction-sections -fdata-sections -fno-tree-loop-optimize -fno-tree-dominator-opts -fno-strength-reduce -fno-stack-protector"
CFLAGS="$OPTFLAGS $OPT47 -ffunction-sections -fdata-sections -fno-stack-protector"
LDFLAGS="-Wl,--gc-sections"

export ac_cv_c_inline="no"

%if %{_CROSS}
%if (%{_arch} == "x86_64") || (%{_arch} == "i386")
export cf_cv_type_of_bool="unsigned char"
export cf_cv_func_nanosleep="yes"
export cf_cv_func_mkstemp="yes"
export cf_cv_working_poll="yes"
%endif
%if (%{_arch} == "x86_64")
export cf_cv_typeof_chtype="int"
%endif
%endif

%configure \
%if %{_CROSS}
	--build=%{_build} \
	--host=%{target} \
%endif
	--enable-big-core \
	--without-shared \
	--with-normal \
	--without-debug \
	--without-cxx \
	--without-cxx-binding \
	--with-progs \
	--without-ada \
	--disable-home-terminfo \
	--with-install-prefix=${RPM_BUILD_ROOT} \
	--with-terminfo-dirs=/usr/share/terminfo \
	--with-default-terminfo-dir=/usr/share/terminfo \
	&>FConfigure_log.txt

make &>FMake_log.txt


# ------------------- Install ----------------------------------------
%Install
PATH=%{csysbindir}:$PATH:/opt/perl/bin
GCC_NoHostPath=d
UCLIBC_GCC_INC_SKIP=%{_includedir}
export PATH GCC_NoHostPath UCLIBC_GCC_INC_SKIP

rm -rf $RPM_BUILD_ROOT
mkdir -p $RPM_BUILD_ROOT
mkdir -p %{_srcrpmdir}
mkdir -p $RPM_BUILD_ROOT%{docdir}
mkdir -p $RPM_BUILD_ROOT/usr/share/terminfo

make install &>Finstall_log.txt

cp ANNOUNCE AUTHORS README $RPM_BUILD_ROOT%{docdir}/

TERMINFO=$RPM_BUILD_ROOT%{_datadir}/terminfo
mkdir -p $TERMINFO
mkdir -p $TERMINFO/l
cp $RPM_BUILD_ROOT/usr/share/terminfo/l/linux* $TERMINFO/l/
mkdir -p $TERMINFO/r
cp $RPM_BUILD_ROOT/usr/share/terminfo/r/rxvt* $TERMINFO/r/

mkdir -p $TERMINFO/v
cp $RPM_BUILD_ROOT/usr/share/terminfo/v/vt100 $TERMINFO/v
cp $RPM_BUILD_ROOT/usr/share/terminfo/v/vt102 $TERMINFO/v
cp $RPM_BUILD_ROOT/usr/share/terminfo/v/vt220 $TERMINFO/v
mkdir -p $TERMINFO/x
cp $RPM_BUILD_ROOT/usr/share/terminfo/x/xterm* $TERMINFO/x/

rm -rf $RPM_BUILD_ROOT/usr/share
rm -rf $RPM_BUILD_ROOT%{_mandir}/man3

pushd $RPM_BUILD_ROOT/%{_libdir}
  mkdir tmp

  tmpZ=0
  for i in *.a; do
    ar -x $i
    rm $i
    tmp=$tmpZ"_"
    for o in *.o; do
      mv $o tmp/$tmp$o
    done
    tmpZ=$(($tmpZ + 1))
  done

  cd tmp
  %{target_}ar -cr libncurses.a *.o
  %{target_}ranlib libncurses.a
  mv libncurses.a ../libncurses.a
  cd ..
  rm -r tmp
popd

sed -i s:" \"\" ":" \"no\" ": $RPM_BUILD_ROOT%{_bindir}/ncurses5-config
sed -i s:"-L\${exec_prefix}/lib  ":: $RPM_BUILD_ROOT%{_bindir}/ncurses5-config

%if %{_CROSS} && %{BUILD_LOCAL_CROSS}
TARGET_PRFIX=%{_prefix}
TARGET_CROSS_PRFIX=%{csysdir}
TARGET_NAME=%{target}
export TARGET_PRFIX TARGET_CROSS_PRFIX TARGET_NAME
%SOURCE1
rm $RPM_BUILD_ROOT%{cross_bindir}/c*
rm $RPM_BUILD_ROOT%{cross_bindir}/i*
rm $RPM_BUILD_ROOT%{cross_bindir}/reset
rm $RPM_BUILD_ROOT%{cross_bindir}/t*
rm -r $RPM_BUILD_ROOT%{cross_mandir}
rm -r $RPM_BUILD_ROOT%{cross_datadir}/tabset
rm -r $RPM_BUILD_ROOT%{cross_datadir}/terminfo
sed -i s:"/usr/uC/":"%{csysdir}/usr/uC/": $RPM_BUILD_ROOT%{cross_bindir}/ncurses5-config
sed -i s:" /usr/include ":" %{cross_includedir} ": $RPM_BUILD_ROOT%{cross_bindir}/ncurses5-config
%endif
sed -i s:" /usr/include ":" %{_includedir} ": $RPM_BUILD_ROOT%{_bindir}/ncurses5-config

%if !%{_CROSS}
if test -f /opt/aldccp/bin/packages_idx; then
  /opt/aldccp/bin/packages_idx -p %{name} -v %{version} -r %{DestDir} $RPM_BUILD_ROOT
fi
%endif

# ------------------- Clean ------------------------------------------
%Clean 

DE=%{version}-%{release}.%{TO_ARCH}.rpm
DD=%{_rpmdir}/%{TO_ARCH}/%{ALD_Ver}%{DestDir}
DEC=%{version}-%{release}.%{_build_cpu}.rpm
DDC=%{_rpmdir}/%{_build_cpu}/%{ALD_Ver}%{CrossDestDir}/%{TO_ARCH}%{DestDir}

pushd %{_rpmdir}/%{_target_cpu}
  SE=%{version}-%{release}.%{_target_cpu}.rpm
  mkdir -p $DD
  mv %{name}-$SE $DD/%{name}-$DE
  mv %{name}-bin-$SE $DD/%{name}-bin-$DE
%if %{BUILD_LOCAL_CROSS}
  mv %{name}-%{TO_ARCH}-$SE $DDC/%{name}-%{TO_ARCH}-$DEC
%endif
popd

# --------------- build *.tgz ------------------------
%if %{BUILD_TAR}
pushd $RPM_BUILD_ROOT
  TMP_TO=$DD/tar
  mkdir -p $TMP_TO
  tmp=%{name}-%{version}-%{release}.%{TO_ARCH}
  mkdir -p $tmp
  cp -r usr $tmp/
  tar -cjf $tmp.tar.bz2 $tmp
  mv $tmp.tar.bz2 $TMP_TO
  rm -r $tmp
%if %{BUILD_LOCAL_CROSS}
  TMP_TO=$DDC/tar
  mkdir -p $TMP_TO
  tmp=%{name}-%{TO_ARCH}-%{version}-%{release}.%{_build_cpu}
  mkdir -p $tmp
  cp -r cross $tmp/
  tar -cjf $tmp.tar.bz2 $tmp
  mv $tmp.tar.bz2 $TMP_TO
  rm -r $tmp
%endif #BUILD_LOCAL_CROSS
popd
%endif  #BUILD_TAR

# ------------------- Files ------------------------------------------
%Files
%{_bindir}/ncurses5-config
%{_includedir}
%{_libdir}/*.a
%{_datadir}/tabset
%{_datadir}/terminfo
%{docdir}

%files bin
%exclude %{_bindir}/ncurses5-config
%{_bindir}
%{_mandir}

%if %{BUILD_LOCAL_CROSS}

%Files %{TO_ARCH}
%{cross_bindir}/ncurses5-config
%{cross_includedir}
%{cross_libdir}
%{cross_docdir}

%endif  #BUILD_LOCAL_CROSS

